@@ -179,6 +179,17 @@ class Auth:
         request_object = requests.post(request_ref, headers=headers, data=data)
         raise_detailed_error(request_object)
         return request_object.json()
+    
+    def update_profile(self, id_token, display_name = None, photo_url = None, delete_attribute = None):
+        """
+        https://firebase.google.com/docs/reference/rest/auth#section-update-profile
+        """
+        request_ref = "https://identitytoolkit.googleapis.com/v1/accounts:update?key={0}".format(self.api_key)
+        headers = {"content-type": "application/json; charset=UTF-8"}
+        data = json.dumps({"idToken": id_token, "displayName": display_name, "photoURL": photo_url, "deleteAttribute": delete_attribute, "returnSecureToken": True})
+        request_object = requests.post(request_ref, headers=headers, data=data)
+        raise_detailed_error(request_object)
+        return request_object.json()
 
 
 class Database:
@@ -376,41 +387,50 @@ class Database:
         return PyreResponse(convert_to_pyre(data), origin.key())
 
     def get_etag(self, token=None, json_kwargs={}):
-         request_ref = self.build_request_url(token)
-         headers = self.build_headers(token)
-         # extra header to get ETag
-         headers['X-Firebase-ETag'] = 'true'
-         request_object = self.requests.get(request_ref, headers=headers)
-         raise_detailed_error(request_object)
-         return request_object.headers['ETag']
+        request_ref = self.build_request_url(token)
+        headers = self.build_headers(token)
+        # extra header to get ETag
+        headers['X-Firebase-ETag'] = 'true'
+        request_object = self.requests.get(request_ref, headers=headers)
+        raise_detailed_error(request_object)
+        return {
+           'ETag': request_object.headers['ETag'],
+           'value': request_object.json()
+        }
 
     def conditional_set(self, data, etag, token=None, json_kwargs={}):
-         request_ref = self.check_token(self.database_url, self.path, token)
-         self.path = ""
-         headers = self.build_headers(token)
-         headers['if-match'] = etag
-         request_object = self.requests.put(request_ref, headers=headers, data=json.dumps(data, **json_kwargs).encode("utf-8"))
+        request_ref = self.check_token(self.database_url, self.path, token)
+        self.path = ""
+        headers = self.build_headers(token)
+        headers['if-match'] = etag
+        request_object = self.requests.put(request_ref, headers=headers, data=json.dumps(data, **json_kwargs).encode("utf-8"))
 
-         # ETag didn't match, so we should return the correct one for the user to try again
-         if request_object.status_code == 412:
-             return {'ETag': request_object.headers['ETag']}
+        # ETag didn't match, so we should return the correct one for the user to try again
+        if request_object.status_code == 412:
+            return {
+               'ETag': request_object.headers['ETag'],
+               'value': request_object.json()
+            }
 
-         raise_detailed_error(request_object)
-         return request_object.json()
+        raise_detailed_error(request_object)
+        return request_object.json()
 
     def conditional_remove(self, etag, token=None):
-         request_ref = self.check_token(self.database_url, self.path, token)
-         self.path = ""
-         headers = self.build_headers(token)
-         headers['if-match'] = etag
-         request_object = self.requests.delete(request_ref, headers=headers)
+        request_ref = self.check_token(self.database_url, self.path, token)
+        self.path = ""
+        headers = self.build_headers(token)
+        headers['if-match'] = etag
+        request_object = self.requests.delete(request_ref, headers=headers)
 
-         # ETag didn't match, so we should return the correct one for the user to try again
-         if request_object.status_code == 412:
-             return {'ETag': request_object.headers['ETag']}
+        # ETag didn't match, so we should return the correct one for the user to try again
+        if request_object.status_code == 412:
+            return {
+               'ETag': request_object.headers['ETag'],
+               'value': request_object.json()
+            }
 
-         raise_detailed_error(request_object)
-         return request_object.json()
+        raise_detailed_error(request_object)
+        return request_object.json()
 
 
 class Storage:
@@ -459,8 +479,17 @@ class Storage:
             raise_detailed_error(request_object)
             return request_object.json()
 
-    def delete(self, name):
-        self.bucket.delete_blob(name)
+    def delete(self, name, token):
+        if self.credentials:
+            self.bucket.delete_blob(name)
+        else:
+            request_ref = self.storage_bucket + "/o?name={0}".format(name)
+            if token:
+                headers = {"Authorization": "Firebase " + token}
+                request_object = self.requests.delete(request_ref, headers=headers)
+            else:
+                request_object = self.requests.delete(request_ref)
+            raise_detailed_error(request_object)
 
     def download(self, path, filename, token=None):
         # remove leading backlash
